/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: ADDIInstruction.java,v 1.3 2009/08/12 15:17:59 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.isa.arithmetic;

import com.unitesk.testfusion.core.exception.UnpredictableBehavior;
import com.unitesk.testfusion.core.model.ContentType;
import com.unitesk.testfusion.core.model.Operand;
import com.unitesk.testfusion.core.model.OperandType;
import com.unitesk.testfusion.core.model.Processor;
import com.unitesk.testfusion.core.model.register.Register;
import com.unitesk.testfusion.core.type.WordType;
import com.unitesk.testfusion.core.util.Utils;
import com.unitesk.testfusion.demo.model.DemoRegister;
import com.unitesk.testfusion.demo.model.exception.IntegerOverflow;
import com.unitesk.testfusion.demo.model.isa.DemoInstruction;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class ADDIInstruction extends DemoInstruction
{
    public ADDIInstruction()
    {
        super("addi");
        
        registerOperand(new Operand(this, "rd", Operand.OUTPUT, OperandType.REGISTER,  ContentType.WORD));
        registerOperand(new Operand(this, "rs", Operand.INPUT,  OperandType.REGISTER,  ContentType.WORD));
        registerOperand(new Operand(this, "im", Operand.INPUT,  OperandType.IMMEDIATE, ContentType.HALF_WORD));
        
        registerException(new IntegerOverflow());
    }
    
    public ADDIInstruction(DemoRegister rd, DemoRegister rs, short im)
    {
        this();
        
        Operand rdOperand = getOperand("rd");
        Operand rsOperand = getOperand("rs");
        Operand imOperand = getOperand("im");
        
        rdOperand.setRegister(rd);
        rsOperand.setRegister(rs);
        imOperand.setShortValue(im);
    }
    
    public boolean isIntegerOverflow(int rsValue, int imValue)
    {
        long rdValue = rsValue + imValue;
        
        return rdValue < Integer.MIN_VALUE || rdValue > Integer.MAX_VALUE;  
    }
    
    @Override
    public void calculate(Processor processor)
    {
        super.calculate(processor);
        
        Operand rdOperand = getOperand("rd");
        Operand rsOperand = getOperand("rs");
        Operand imOperand = getOperand("im");

        int rsValue = rsOperand.getIntegerValue();
        int imValue = imOperand.getShortValue();

        if(!WordType.isWord(rsValue))
            { setException(new UnpredictableBehavior()); return; }
        
        if(isIntegerOverflow(rsValue, imValue))
            { setException(new IntegerOverflow()); return; }
        
        rdOperand.setIntegerValue(rsValue + imValue);
    }

    @Override
    public void execute(Processor processor)
    {
        super.execute(processor);
        
        Operand rdOperand = getOperand("rd");
        Operand rsOperand = getOperand("rs");
        Operand imOperand = getOperand("im");
        
        Register rsRegister = rsOperand.getRegister();
        Register rdRegister = rdOperand.getRegister();
        
        int rsValue = rsRegister.getIntegerValue();
        int imValue = imOperand.getShortValue();
        
        if(!WordType.isWord(rsValue))
            { setException(new UnpredictableBehavior()); return; }
    
        if(isIntegerOverflow(rsValue, imValue))
            { setException(new IntegerOverflow()); return; }
        
        rdRegister.setIntegerValue(rsValue + imValue);
    }
    
    @Override
    public String toString()
    {
        Operand rdOperand = getOperand("rd");
        Operand rsOperand = getOperand("rs");
        Operand imOperand = getOperand("im");
        
        return "addi " + rdOperand.getRegister() + ", "  + rsOperand.getRegister() + ", 0x" + Utils.toHexString(imOperand.getShortValue());
    }
    
    @Override
    public ADDIInstruction createInstruction()
    {
        return new ADDIInstruction();
    }
}
